@@ -1,5 +1,5 @@
 # Lazygal, a lazy satic web gallery generator.
-# Copyright (C) 2007-2010 Alexandre Rossi <alexandre.rossi@gmail.com>
+# Copyright (C) 2007-2011 Alexandre Rossi <alexandre.rossi@gmail.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@ import locale
 import gc
 
 import make
-import sourcetree, tpl, feeds, newsize, metadata, mediautils
+import sourcetree, tpl, newsize, metadata, mediautils
 import genpage, genmedia, genfile
 
 from sourcetree import SOURCEDIR_CONFIGFILE
@@ -169,7 +169,7 @@ class SubgalBreak(make.MakeTask):
 class WebalbumMediaTask(make.GroupTask):
 
     def __init__(self, webgal, media, album):
-        make.MakeTask.__init__(self)
+        super(WebalbumMediaTask, self).__init__()
 
         self.album = album
         self.webgal = webgal
@@ -213,20 +213,26 @@ class WebalbumImageTask(WebalbumMediaTask):
     """
 
     def __init__(self, webgal, image, album):
-        WebalbumMediaTask.__init__(self, webgal, image, album)
+        super(WebalbumImageTask, self).__init__(webgal, image, album)
 
         self.thumb = genmedia.ImageOtherSize(self.webgal, self.media,
                                              genmedia.THUMB_SIZE_NAME)
         self.add_dependency(self.thumb)
 
+    def get_original_or_symlink(self):
+        if not self.album.orig_symlink:
+            return genfile.ImageOriginal(self.webgal, self.media)
+        else:
+            return genfile.SymlinkImageOriginal(self.webgal, self.media)
+
     def get_original(self):
         if not self.original:
-            self.original = genfile.ImageOriginal(self.webgal, self.media)
+            self.original = self.get_original_or_symlink()
         return self.original
 
     def get_resized(self, size_name):
         if self.album.browse_size_strings[size_name] == '0x0':
-            return genfile.ImageOriginal(self.webgal, self.media)
+            return self.get_original_or_symlink()
         else:
             return genmedia.ImageOtherSize(self.webgal, self.media, size_name)
 
@@ -242,7 +248,7 @@ class WebalbumVideoTask(WebalbumMediaTask):
     def __init__(self, webgal, video, album):
         self.webvideo = None
 
-        WebalbumMediaTask.__init__(self, webgal, video, album)
+        super(WebalbumVideoTask, self).__init__(webgal, video, album)
 
         self.thumb = None # none yet
 
@@ -265,12 +271,12 @@ class WebalbumDir(make.FileMakeObject):
     This is a built web gallery with its files, thumbs and reduced pics.
     """
 
-    def __init__(self, dir, subgals, album, album_dest_dir, clean_dest):
-        self.__mtime = None
-
+    def __init__(self, dir, subgals, album, album_dest_dir, clean_dest=False):
         self.source_dir = dir
         self.path = os.path.join(album_dest_dir, self.source_dir.strip_root())
-        make.FileMakeObject.__init__(self, self.path)
+        if self.path.endswith('/'): self.path = os.path.dirname(self.path)
+
+        super(WebalbumDir, self).__init__(self.path)
 
         self.add_dependency(self.source_dir)
         self.subgals = subgals
@@ -278,10 +284,6 @@ class WebalbumDir(make.FileMakeObject):
 
         self.flattening_dir = None
 
-        # mtime for directories must be saved, because the WebalbumDir gets
-        # updated as its dependencies are built.
-        self.__mtime = self.get_mtime()
-
         self.clean_dest = clean_dest
 
         # Create the directory if it does not exist
@@ -290,6 +292,8 @@ class WebalbumDir(make.FileMakeObject):
                            % self.source_dir.strip_root(), 'info')
             self.album.log("(%s)" % self.path)
             os.makedirs(self.path, mode = 0755)
+            # Directory did not exist, mark it as so
+            self.stamp_delete()
 
         self.medias = []
         self.sort_task = SubgalSort(self)
@@ -302,7 +306,7 @@ class WebalbumDir(make.FileMakeObject):
             elif media.type == 'video':
                 media_task = WebalbumVideoTask(self, media, self.album)
             else:
-                raise NotImplementedError("Unknonwn media type '%s'"\
+                raise NotImplementedError("Unknown media type '%s'"\
                                           % media.type)
             self.medias.append(media_task)
             self.add_dependency(media_task)
@@ -311,7 +315,9 @@ class WebalbumDir(make.FileMakeObject):
 
         if not self.should_be_flattened():
             self.break_task = SubgalBreak(self)
-            self.add_dependency(self.break_task)
+            # This task is special because it populates dependencies. This is
+            # why it needs to be built before a build check.
+            self.break_task.make()
 
             if self.album.thumbs_per_page > 0:
                 # FIXME: If pagination is 'on', galleries need to be sorted
@@ -321,6 +327,8 @@ class WebalbumDir(make.FileMakeObject):
 
             self.webgal_pic = genmedia.WebalbumPicture(self)
             self.add_dependency(self.webgal_pic)
+        else:
+            self.break_task = None
 
     def add_index_page(self, subgals, galleries):
         page_number = self.break_task.next_page_number()
@@ -329,9 +337,10 @@ class WebalbumDir(make.FileMakeObject):
                                              subgals, galleries)
             self.add_dependency(page)
 
-    def get_mtime(self):
-        # Use the saved mtime that was initialized once, in self.__init__()
-        return self.__mtime or super(WebalbumDir, self).get_mtime()
+    def register_output(self, output):
+        # We only care about output in the current directory
+        if os.path.dirname(output) == self.path:
+            super(WebalbumDir, self).register_output(output)
 
     def get_subgal_count(self):
         if self.flatten_below():
@@ -351,9 +360,10 @@ class WebalbumDir(make.FileMakeObject):
             all_medias.extend(subgal.get_all_medias_tasks())
         return all_medias
 
-    def should_be_flattened(self):
+    def should_be_flattened(self, path=None):
+        if path is None: path = self.source_dir.path
         return self.album.dir_flattening_depth is not False\
-        and self.source_dir.get_album_level() > self.album.dir_flattening_depth
+        and self.source_dir.get_album_level(path) > self.album.dir_flattening_depth
 
     def flatten_below(self):
         if self.album.dir_flattening_depth is False:
@@ -365,7 +375,30 @@ class WebalbumDir(make.FileMakeObject):
         else:
             return False
 
-    def build(self):
+    def rel_path(self, target_srcdir_path):
+        '''
+        Returns the relative path to go from this directory to
+        target_srcdir_path.
+        '''
+        return self.source_dir.rel_path(self.source_dir.path,
+                                        target_srcdir_path)
+
+    def flattening_srcpath(self, srcdir_path):
+        '''
+        Returns the source path in which srcdir_path should flattened, that is
+        the path of the gallery index that will point to srcdir_path's pictures.
+        '''
+        if self.should_be_flattened(srcdir_path):
+            cur_path = srcdir_path
+            while self.should_be_flattened(cur_path):
+                cur_path, dummy = os.path.split(cur_path)
+            return cur_path
+        else:
+            return ''
+
+    def list_foreign_files(self):
+        foreign_files = []
+
         # Check dest for junk files
         extra_files = []
         if self.source_dir.is_album_root():
@@ -382,30 +415,37 @@ class WebalbumDir(make.FileMakeObject):
             if dest_file not in self.output_items and\
                dest_file not in expected_dirs and\
                dest_file not in extra_files:
-                text = ''
-                if self.clean_dest and not os.path.isdir(dest_file):
-                    os.unlink(dest_file)
-                    text = ""
-                else:
-                    text = _("you should")
-                self.album.log(_("  %s RM %s") % (text, dest_file), 'info')
+                foreign_files.append(dest_file)
+
+        return foreign_files
+
+    def build(self):
+        for dest_file in self.list_foreign_files():
+            text = ''
+            if self.clean_dest and not os.path.isdir(dest_file):
+                os.unlink(dest_file)
+                text = ""
+            else:
+                text = _("you should")
+            self.album.log(_("  %s RM %s") % (text, dest_file), 'info')
 
     def make(self, force=False):
-        make.FileMakeObject.make(self, force)
+        needed_build = self.needs_build()
+
+        super(WebalbumDir, self).make(force or needed_build)
 
         # Although we should have modified the directory contents and thus its
         # mtime, it is possible that the directory mtime has not been updated
         # if we regenerated without adding/removing pictures (to take into
         # account a rotation for example). This is why we force directory mtime
-        # update here.
-        os.utime(self.path, None)
+        # update here if something has been built.
+        if needed_build: os.utime(self.path, None)
 
 
 class SharedFiles(make.FileSimpleDependency):
 
     def __init__(self, album, dest_dir):
         self.path = os.path.join(dest_dir, DEST_SHARED_DIRECTORY_NAME)
-        make.FileSimpleDependency.__init__(self, self.path)
 
         # Create the shared files directory if it does not exist
         if not os.path.isdir(self.path):
@@ -413,6 +453,8 @@ class SharedFiles(make.FileSimpleDependency):
             album.log("(%s)" % self.path)
             os.makedirs(self.path, mode = 0755)
 
+        super(SharedFiles, self).__init__(self.path)
+
         for shared_file in glob.glob(\
           os.path.join(album.tpl_dir, THEME_SHARED_FILE_PREFIX + '*')):
             shared_file_name = os.path.basename(shared_file).\
@@ -432,7 +474,8 @@ class SharedFiles(make.FileSimpleDependency):
 
 class Album:
 
-    def __init__(self, source_dir, thumb_size_string, browse_size_strings,
+    def __init__(self, source_dir,
+                 thumb_size_string='150x113', browse_size_strings=None,
                  optimize=False, progressive=False, quality=85,
                  dir_flattening_depth=False, thumbs_per_page=0,
                  dirzip=False,
@@ -444,19 +487,26 @@ class Album:
         self.source_dir = self.source_dir.decode(sys.getfilesystemencoding())
 
         self.thumb_size_string = thumb_size_string
-        self.browse_size_strings = dict(browse_size_strings)
+        if browse_size_strings is not None:
+            self.browse_size_strings = dict(browse_size_strings)
+            self.default_size_name = browse_size_strings[0][0]
+        else:
+            self.browse_size_strings = {'small': '800x600',
+                                        'medium': '1024x768'}
+            self.default_size_name = 'small'
+
         self.newsizers = {}
         for size_name, size_string in self.browse_size_strings.items():
             self.newsizers[size_name] = newsize.get_newsizer(size_string)
         self.newsizers[genmedia.THUMB_SIZE_NAME] = newsize.get_newsizer(self.thumb_size_string)
 
-        self.default_size_name = browse_size_strings[0][0]
         self.quality = quality
 
         self.tpl_loader = None
         self.tpl_vars = {}
         self.original = False
         self.orig_base = None
+        self.orig_symlink = False
         self.thumbs_per_page = thumbs_per_page
         self.dir_flattening_depth = dir_flattening_depth
         self.dirzip = dirzip
@@ -472,6 +522,8 @@ class Album:
         self.transcoder = None
         self.videothumb = None
 
+        self.set_webalbumpic()
+
     def set_theme(self, theme=tpl.DEFAULT_TEMPLATE, default_style=None):
         self.theme = theme
 
@@ -494,8 +546,11 @@ class Album:
         # Load styles templates
         for style in styles:
             style_filename = style['filename']
-            if self.tpl_loader.is_known_template_type(style_filename):
+            try:
                 self.tpl_loader.load(style_filename)
+            except ValueError:
+                # Not a known emplate ext, ignore
+                pass
 
     def get_transcoder(self):
         if self.transcoder is None:
@@ -527,9 +582,10 @@ class Album:
             styles.append(style)
         return styles
 
-    def set_original(self, original=False, orig_base=None):
+    def set_original(self, original=False, orig_base=None, orig_symlink=False):
         self.original = original
-        if self.original and orig_base:
+        self.orig_symlink = orig_symlink
+        if self.original and orig_base and not orig_symlink:
             self.orig_base = orig_base
         else:
             self.orig_base = None
@@ -590,13 +646,15 @@ class Album:
             head, tail = os.path.split(head)
         return False
 
-    def generate_default_medatada(self):
+    def generate_default_metadata(self):
         '''
         Generate default metada files if no exists.
         '''
         self.log(_("Generating metadata in %s") % self.source_dir)
 
         for root, dirnames, filenames in os.walk(self.source_dir):
+            filenames.sort() # This is required for the ignored files
+                             # checks to be reliable.
             source_dir = sourcetree.Directory(root, [], filenames, self)
             self.log(_("[Entering %%ALBUMROOT%%/%s]") % source_dir.strip_root(),
                      'info')
@@ -669,8 +727,11 @@ class Album:
 
             if feed:
                 feed.push_dir(destgal)
-            if destgal.needs_build() or check_all_dirs:
+
+            if check_all_dirs:
                 destgal.make()
+            elif destgal.needs_build():
+                destgal.make(force=True) # avoid another needs_build() call in make()
             else:
                 self.log(_("  SKIPPED because of mtime, touch source or use --check-all-dirs to override."))
 
