@@ -125,6 +125,44 @@ class WebalbumPage(genfile.WebalbumFile):
 
         return osize_index_links
 
+    def _get_webgal_id(self, srcdir_path):
+        if self.dir.should_be_flattened(srcdir_path):
+            rawid = self.dir.source_dir.rel_path(self.dir.flattening_srcpath(srcdir_path), srcdir_path)
+        else:
+            rawid = os.path.basename(srcdir_path)
+        return rawid.replace(' /\\', '_')
+
+    def _get_webgal_link(self, srcdir_path):
+        link_target = self._add_size_qualifier('index.html', self.size_name)
+
+        if self.dir.should_be_flattened(srcdir_path):
+            # Add anchor target to get straight to gallery listing
+            link_target = link_target + '#' + self._get_webgal_id(srcdir_path)
+
+        # Add relative path to link if needed
+        index_path = None
+        if self.dir.should_be_flattened(srcdir_path):
+            index_path = self.dir.flattening_srcpath(srcdir_path)
+        if srcdir_path != self.dir.source_dir.path:
+            index_path = srcdir_path
+        if index_path is not None:
+            link_target = self.dir.rel_path(index_path) + link_target
+
+        return self.url_quote(link_target)
+
+    def _gen_webgal_path(self):
+        wg_path = []
+        for dirmd in self.dir.source_dir.parents_metadata():
+            wg = {}
+            wg['link'] = self._get_webgal_link(dirmd.dir_path)
+            wg['name'] = dirmd.get_title()
+            wg['root'] = dirmd.dir_path == self.dir.album.source_dir
+            wg['current'] = dirmd.dir_path == self.dir.source_dir.path
+            wg_path.append(wg)
+
+        wg_path.reverse()
+        return wg_path
+
     def _add_size_qualifier(self, path, size_name):
         return self.dir.album._add_size_qualifier(path, size_name)
 
@@ -132,7 +170,7 @@ class WebalbumPage(genfile.WebalbumFile):
         return genshi.core.Markup(value)
 
     def url_quote(self, url):
-        return urllib.quote(url.encode(sys.getfilesystemencoding()), safe=':/')
+        return urllib.quote(url.encode(sys.getfilesystemencoding()), safe=':/#')
 
 
 class WebalbumBrowsePage(WebalbumPage):
@@ -149,6 +187,10 @@ class WebalbumBrowsePage(WebalbumPage):
         # Depends on source directory in case an image was deleted
         self.add_dependency(self.dir.source_dir)
 
+        # Depend on the comment file if it exists.
+        if self.webalbum_media.media.comment_file_path is not None:
+            self.add_file_dependency(self.webalbum_media.media.comment_file_path)
+
         self.add_dependency(self.dir.sort_task)
 
         self.set_template('browse.thtml')
@@ -160,6 +202,10 @@ class WebalbumBrowsePage(WebalbumPage):
         self.dir.album.log("(%s)" % self.page_path)
 
         tpl_values = {}
+
+        # Breadcrumbs
+        tpl_values['webgal_path'] = self._gen_webgal_path()
+
         tpl_values['name'] = self.media.filename
         tpl_values['mediatype'] = self.media.type
         tpl_values['dir'] = self.dir.source_dir.strip_root()
@@ -232,7 +278,11 @@ class WebalbumImagePage(WebalbumBrowsePage):
             tpl_values['iso'] = image_info.get_iso()
             tpl_values['fnumber'] = image_info.get_fnumber()
             tpl_values['focal_length'] = image_info.get_focal_length()
-            tpl_values['comment'] = image_info.get_comment()
+            comment = image_info.get_comment()
+            if comment == '' or comment is None:
+                tpl_values['comment'] = None
+            else:
+                tpl_values['comment'] = self._do_not_escape(comment)
 
 
 class WebalbumVideoPage(WebalbumBrowsePage):
@@ -261,7 +311,6 @@ class WebalbumIndexPage(WebalbumPage):
             self.add_dependency(dir.source_dir.metadata)
             if dir is not self.dir:
                 dir.flattening_dir = self.dir
-                self.add_dependency(dir)
 
             self.add_dependency(dir.source_dir)
             self.add_dependency(dir.sort_task)
@@ -276,6 +325,9 @@ class WebalbumIndexPage(WebalbumPage):
             if self.dir.album.dirzip and dir.source_dir.get_media_count() > 1:
                 self.add_dependency(genfile.WebalbumArchive(dir))
 
+        for subgal in self.subgals:
+            self.add_dependency(subgal.source_dir)
+
         self.set_template('dirindex.thtml')
 
     def _get_paginated_name(self, page_number=None):
@@ -334,6 +386,8 @@ class WebalbumIndexPage(WebalbumPage):
         dir_info['image_count'] = dir.source_dir.get_media_count('image')
         dir_info['subgal_count'] = len(dir.source_dir.subdirs)
 
+        dir_info['id'] = self.url_quote(self._get_webgal_id(dir.source_dir.path))
+
         return dir_info
 
     def _get_subgal_links(self):
@@ -355,6 +409,10 @@ class WebalbumIndexPage(WebalbumPage):
 
         values = {}
 
+        # Breadcrumbs (current is static, see dirindex.thtml, that's why the
+        # last item of the list is removed).
+        values['webgal_path'] = self._gen_webgal_path()[:-1]
+
         if not self.dir.source_dir.is_album_root():
             # Parent index link not for album root
             values['parent_index_link'] = self._get_related_index_fn()
@@ -396,7 +454,7 @@ class WebalbumFeed(make.FileMakeObject):
         self.path = os.path.join(dest_dir, 'index.xml')
         make.FileMakeObject.__init__(self, self.path)
         self.feed = feeds.RSS20(self.pub_url)
-        self.item_template = self.album.templates['feeditem.thtml']
+        self.item_template = self.album.tpl_loader.load('feeditem.thtml')
 
     def set_title(self, title):
         self.feed.title = title
