@@ -32,9 +32,9 @@ class MakeTask(object):
     def __init__(self):
         self.deps = []
         self.output_items = []
-        self.__last_build_time = -1 # older than oldest epoch
-        self.__built_once = False
+        self.stamp_delete()
         self.__dep_only = False
+        self.update_build_status()
 
     def add_dependency(self, dependency):
         if self in dependency.deps:
@@ -67,19 +67,45 @@ class MakeTask(object):
         self.__last_build_time = build_time
         self.__built_once = True
 
-    def needs_build(self):
-        if not self.__built_once:
-            return True
+    def stamp_delete(self):
+        self.__last_build_time = -1 # older than oldest epoch
+        self.__built_once = False
+
+    def built_once(self):
+        return self.__built_once
+
+    def update_build_status(self):
+        '''
+        Do stuff to update the build status (e.g. probe filesystem, deps).
+        '''
+        pass
+
+    def needs_build(self, return_culprit=False):
+        if not self.built_once():
+            if return_culprit:
+                return 'never built'
+            else:
+                return True
 
         for dependency in self.deps:
             if dependency.get_mtime() > self.get_mtime()\
             or dependency.needs_build():
                 if not dependency.is_dep_only():
-                    return True
+                    if return_culprit:
+                        mtime_gap = dependency.get_mtime() - self.get_mtime()
+                        if mtime_gap > 0:
+                            reason = 'dep newer by %s s' % mtime_gap
+                        elif dependency.needs_build():
+                            reason = dependency.needs_build(True)
+                        else:
+                            raise RuntimeError # should never go here
+                        return dependency, reason
+                    else:
+                        return True
         return False
 
     def make(self, force=False):
-        if self.needs_build() or force:
+        if force or self.needs_build():
             for d in self.deps:
                 d.make() # dependency building not forced
             self.call_build()
@@ -90,7 +116,11 @@ class MakeTask(object):
         overridden with more complicated things in subclasses. The purpose is
         to setup some state before and/or after build.
         """
-        self.build()
+        try:
+            self.build()
+        except KeyboardInterrupt:
+            self.clean_output()
+            sys.exit(1)
         self.stamp_build()
 
     def build(self):
@@ -105,8 +135,29 @@ class MakeTask(object):
         This provides a facility to register within the makefile machinery what
         items are built from the task.
         """
-        if not output in self.output_items:
-            self.output_items.append(output)
+        self.output_items.append(output)
+
+    def clean_output(self, output):
+        """
+        Clean-up in case of interruption (KeyboardInterrupt).
+        """
+        pass
+
+    def print_dep_entry(self, level=0):
+        indent = ''
+        for index in range(0, level):
+            indent = indent + '\t'
+
+        print indent, self, self.get_mtime()
+
+    def print_dep_tree(self, depth=1, parent_level=-1):
+        level = parent_level+1
+        if level > depth: return
+
+        self.print_dep_entry(level)
+
+        for d in self.deps:
+            d.print_dep_tree(depth, level)
 
 
 class GroupTask(MakeTask):
@@ -114,58 +165,70 @@ class GroupTask(MakeTask):
     A class that builds nothing but groups subtasks.
     """
 
-    def build(self):
-        pass
+    def built_once(self):
+        return True # GroupTask is all about the deps.
+
+    def update_build_status(self):
+        super(GroupTask, self).update_build_status()
+        # Find youngest dep, which should indicate latest build.
+        mtime = None
+        for dependency in self.deps:
+            if dependency.built_once():
+                new_mtime = dependency.get_mtime()
+                if mtime is None or new_mtime > mtime:
+                    mtime = new_mtime
+        if mtime is None:
+            self.stamp_delete()
+        else:
+            self.stamp_build(mtime)
 
     def add_dependency(self, dependency):
         super(GroupTask, self).add_dependency(dependency)
-        if dependency.get_mtime() > self.get_mtime():
-            self.stamp_build(dependency.get_mtime())
+        dep_mtime = dependency.get_mtime()
+        if dep_mtime > self.get_mtime():
+            self.stamp_build(dep_mtime)
+
+    def build(self):
+        pass
 
 
-class FileSimpleDependency(MakeTask):
+class FileMakeObject(MakeTask):
+
+    def __init__(self, path):
+        self._path = path
+        super(FileMakeObject, self).__init__()
+        self.register_output(self._path)
+
+    def update_build_status(self):
+        super(FileMakeObject, self).update_build_status()
+        # Update build info according to file existence
+        if os.path.exists(self._path):
+            self.stamp_build(os.path.getmtime(self._path))
+        else:
+            self.stamp_delete()
+
+    def clean_output(self):
+        print "os.unlink(" + self._path + ")"
+        os.unlink(self._path)
+
+    def __repr__(self):
+        return '%s(%s)' % (self.__class__.__name__, self._path.encode('utf-8'))
+
+
+class FileSimpleDependency(FileMakeObject):
     """
     Simple file dependency that needn't build. It just should be there.
     """
 
     def __init__(self, path):
-        MakeTask.__init__(self)
-        self._path = path
-        try:
-            mtime = self.get_mtime()
-        except OSError:
-            pass
-        else:
-            self.stamp_build(mtime)
-
-    def get_mtime(self):
-        return os.path.getmtime(self._path)
+        super(FileSimpleDependency, self).__init__(path)
+        assert self.built_once()
 
     def build(self):
         pass
 
-
-class FileMakeObject(FileSimpleDependency):
-
-    def __init__(self, path):
-        FileSimpleDependency.__init__(self, path)
-        self.register_output(self._path)
-
-    def needs_build(self):
-        return FileSimpleDependency.needs_build(self)\
-               or not os.path.exists(self._path)
-
-    def get_mtime(self):
-        try:
-            mtime = super(FileMakeObject, self).get_mtime()
-        except OSError:
-            # Return last built time
-            mtime = super(FileSimpleDependency, self).get_mtime()
-        return mtime
-
-    def build(self):
-        MakeTask.build(self)
-
+    def clean_output(self):
+        pass
 
 class FileCopy(FileMakeObject):
     """
@@ -182,4 +245,21 @@ class FileCopy(FileMakeObject):
         shutil.copy(self.src, self.dst)
 
 
+class FileSymlink(FileMakeObject):
+    """
+    Simple file symlink make target.
+    """
+
+    def __init__(self, src, dst):
+        self.src = src
+        self.dst = dst
+        FileMakeObject.__init__(self, dst)
+        self.add_file_dependency(self.src)
+
+    def build(self):
+        if os.access(self.dst, os.F_OK):
+            os.remove(self.dst)
+        os.symlink(self.src, self.dst)
+
+
 # vim: ts=4 sw=4 expandtab
