@@ -1,5 +1,5 @@
 # Lazygal, a lazy satic web gallery generator.
-# Copyright (C) 2007-2010 Alexandre Rossi <alexandre.rossi@gmail.com>
+# Copyright (C) 2007-2011 Alexandre Rossi <alexandre.rossi@gmail.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,12 +15,11 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-import os, sys, datetime, locale
+import os, locale
 import codecs
 import Image
 
-import pyexiv2
-
+from lazygal import pyexiv2api as pyexiv2
 from lazygal import make
 
 
@@ -31,11 +30,51 @@ MATEW_TAGS = {
 }
 MATEW_METADATA = 'album_description'
 
+FILE_METADATA = ('album-name', 'album-description', 'album-picture', )
+FILE_METADATA_MEDIA_SUFFIX = '.comment'
 
-class _ImageInfoTags(object):
+
+class FileMetadata(object):
+
+    def __init__(self, path):
+        self.path = path
+
+    def contents(self, splitter=None):
+        try:
+            with codecs.open(self.path, 'r',
+                             locale.getpreferredencoding()) as f:
+
+                # Not sure why codecs.open() does not skip the UTF-8 BOM. Maybe
+                # this is because the BOM is not required and utf-8-sig handles
+                # this in a better way. Anyway, the following code skips the
+                # UTF-8 BOM if it is present.
+                maybe_bom = f.read(1).encode(locale.getpreferredencoding())
+                if maybe_bom != codecs.BOM_UTF8: f.seek(0)
+
+                c = f.read()
+        except IOError:
+            return None
+
+        if splitter is not None:
+            return map(lambda s: s.strip('\n '), c.split(splitter))
+        else:
+            return c.strip('\n ')
+
+
+class ImageInfoTags(object):
 
     def __init__(self, image_path):
         self.image_path = image_path
+        self._metadata = pyexiv2.ImageMetadata(self.image_path)
+        self._metadata.read()
+
+    def get_tag_value(self, name):
+        if name.startswith('Iptc.'):
+            # Iptc tags are always lists, so, for now, return the first
+            # element.
+            return self._metadata[name].values[0]
+        else:
+            return self._metadata[name].value
 
     def get_date(self):
         '''
@@ -43,17 +82,22 @@ class _ImageInfoTags(object):
         as those were filled by camera, Image DateTime can be update by
         software when editing photos later.
         '''
+        date = None
         try:
-            return self.get_tag_value('Exif.Photo.DateTimeDigitized')
+            date = self.get_tag_value('Exif.Photo.DateTimeDigitized')
         except (IndexError, ValueError, KeyError):
             try:
-                return self.get_tag_value('Exif.Photo.DateTimeOriginal')
+                date = self.get_tag_value('Exif.Photo.DateTimeOriginal')
             except (IndexError, ValueError, KeyError):
                 try:
-                    return self.get_tag_value('Exif.Image.DateTime')
+                    date = self.get_tag_value('Exif.Image.DateTime')
                 except (IndexError, ValueError, KeyError):
                     # No date available in EXIF
-                    return None
+                    pass
+        # Some pics give a zeroed date which is not recognized by pyexiv2 as a
+        # date.
+        if date == '0000:00:00 00:00:00': date = None
+        return date
 
     def get_required_rotation(self):
         try:
@@ -156,45 +200,19 @@ class _ImageInfoTags(object):
             return encoded_string.decode(encoding, 'replace')
 
     def get_exif_usercomment(self):
-        ret = self.get_exif_string('Exif.Photo.UserComment')
-        # This field can contain charset information
-        # FIXME : All this stuff should really go in pyexiv2.
-        if ret.startswith('charset='):
-            tokens = ret.split(' ')
-            csetfield = tokens[0]
-            text = ' '.join(tokens[1:])
-            ignore, cset = csetfield.split('=')
-            cset = cset.strip('"')
-        else:
-            cset = None
-            text = ret
+        return self.get_tag_value('Exif.Photo.UserComment').strip(' \0\x00')
 
-        if cset == 'Unicode':
-            im = Image.open(self.image_path)
-            endian = im.app['APP1'][6:8]
-            if endian == 'MM':
-                encoding = 'utf-16be'
-            elif endian == 'II':
-                encoding = 'utf-16le'
-            else:
-                raise ValueError
-
-        elif cset == 'Ascii':
-            encoding = 'ascii'
-        elif cset == 'Jis':
-            encoding = 'shift_jis'
-        else:
-            # Fallback to utf-8 as this is mostly the default for Linux
-            # distributions.
-            encoding = 'utf-8'
-
-        return self._fallback_to_encoding(text, encoding).strip('\0')
+    def get_file_comment(self):
+        fmd = FileMetadata(self.image_path + FILE_METADATA_MEDIA_SUFFIX)
+        return fmd.contents()
 
     def get_comment(self):
         try:
-            ret = self.get_exif_usercomment()
-            if ret == '':
-                raise ValueError
+            ret = self.get_file_comment()
+            if ret is None:
+                ret = self.get_exif_usercomment()
+                if ret == '':
+                    raise ValueError
         except (ValueError, KeyError):
             try:
                 ret = self.get_exif_string('Exif.Image.ImageDescription')
@@ -286,103 +304,11 @@ class _ImageInfoTags(object):
 
         return flen
 
-
-class _Tags_PyExiv2_Legacy(_ImageInfoTags):
-    """
-    Wrapper for pyexiv2 0.1
-    """
-
-    def __init__(self, image_path):
-        _ImageInfoTags.__init__(self, image_path)
-
-        self._metadata = pyexiv2.Image(image_path.encode(sys.getfilesystemencoding()))
-        self._metadata.readMetadata()
-
-    def get_exif_date(self, name):
-        '''
-        Parses date from EXIF information.
-        '''
-        exif_date = str(self._metadata[name])
-        date, time = exif_date.split(' ')
-        year, month, day = date.split('-')
-        hour, minute, second = time.split(':')
-        return datetime.datetime(int(year), int(month), int(day),
-                                 int(hour), int(minute), int(second))
-
-    def get_int(self, name):
-        return int(self._metadata[name])
-
-    def get_interpreted_value(self, name):
-        return self._metadata.interpretedExifValue(name)
-
-    def get_decoded_utf8(self, name):
-        return self.get_interpreted_value(name).decode('utf-8')
-
-    TAG_PYTRANSLATORS = {
-        'Exif.Photo.DateTimeDigitized' : 'get_exif_date',
-        'Exif.Photo.DateTimeOriginal'  : 'get_exif_date',
-        'Exif.Image.DateTime'          : 'get_exif_date',
-        'Exif.Image.Orientation'       : 'get_int',
-        'Exif.Pentax.LensType'         : 'get_interpreted_value',
-        'Exif.Nikon3.Lens'             : 'get_interpreted_value',
-        'Exif.Nikon3.LensType'         : 'get_interpreted_value',
-        'Exif.Minolta.LensID'          : 'get_interpreted_value',
-        'Exif.Photo.Flash'             : 'get_decoded_utf8',
-    }
-
-    def get_tag_value(self, name, raw=False):
-        if not raw and name in _Tags_PyExiv2_Legacy.TAG_PYTRANSLATORS.keys():
-            translator = getattr(self.__class__,
-                                 _Tags_PyExiv2_Legacy.TAG_PYTRANSLATORS[name])
-            return translator(self, name)
-        else:
-            return self._metadata[name]
-
-    def get_jpeg_comment(self):
-        try:
-            return self._fallback_to_encoding(self._metadata.getComment())
-        except (KeyError, AttributeError):
-            return ''
-
-
-class _Tags_PyExiv2(_ImageInfoTags):
-    """
-    Wrapper for pyexiv2 0.2
-    """
-
-    def __init__(self, image_path):
-        _ImageInfoTags.__init__(self, image_path)
-
-        self._metadata = pyexiv2.ImageMetadata(image_path)
-        self._metadata.read()
-
-    def get_tag_value(self, name, raw=False):
-        if raw:
-            return self._metadata[name].raw_value
-        else:
-            return self._metadata[name].value
-
     def get_jpeg_comment(self):
         try:
             return self._fallback_to_encoding(self._metadata.comment)
         except AttributeError:
-            try:
-                # comment appeared in pyexiv2 0.2.2, so use PIL if this does
-                # not work.
-                im = Image.open(self.image_path)
-                return self._fallback_to_encoding(im.app['COM'])
-            except (KeyError, AttributeError):
-                return ''
-
-
-if 'ImageMetadata' in dir(pyexiv2):
-    # pyexiv2 0.2
-    ImageInfoTags = _Tags_PyExiv2
-elif 'Image' in dir(pyexiv2):
-    # pyexiv2 0.1
-    ImageInfoTags = _Tags_PyExiv2_Legacy
-else:
-    raise ImportError('Unrecognized pyexiv2 version.')
+            return ''
 
 
 class NoMetadata(Exception):
@@ -392,20 +318,27 @@ class NoMetadata(Exception):
     pass
 
 
-class DirectoryMetadata(make.FileSimpleDependency):
+class DirectoryMetadata(make.GroupTask):
 
-    def __init__(self, dir):
-        self.dir = dir
-        self.directory_path = self.dir.path
-        make.FileSimpleDependency.__init__(self, self.directory_path)
+    def __init__(self, dir_path):
+        super(DirectoryMetadata, self).__init__()
 
-        self.description_filename = os.path.join(self.directory_path, MATEW_METADATA)
+        self.dir_path = dir_path
+        self.add_file_dependency(self.dir_path)
+
+        self.description_filename = os.path.join(self.dir_path, MATEW_METADATA)
         if os.path.isfile(self.description_filename):
             self.description_file = self.description_filename
             self.add_file_dependency(self.description_filename)
         else:
             self.description_file = None
 
+            # Add dependency to "file metadata" files if they exist.
+            for file_md_fn in FILE_METADATA:
+                file_md_path = os.path.join(self.dir_path, file_md_fn)
+                if os.path.isfile(file_md_path):
+                    self.add_file_dependency(file_md_path)
+
     def get_matew_metadata(self, metadata, subdir = None):
         '''
         Return dictionary with meta data parsed from Matew like format.
@@ -413,7 +346,7 @@ class DirectoryMetadata(make.FileSimpleDependency):
         if subdir is None:
             path = self.description_file
         else:
-            path = os.path.join(self.directory_path, subdir, MATEW_METADATA)
+            path = os.path.join(self.dir_path, subdir, MATEW_METADATA)
 
         if path is None or not os.path.exists(path):
             raise NoMetadata(_('Could not open metadata file %s') % path)
@@ -434,13 +367,37 @@ class DirectoryMetadata(make.FileSimpleDependency):
                     if tag == 'album_picture':
                         if subdir is not None:
                             data = os.path.join(subdir, data)
-                        data = os.path.join(self.dir.path, data)
+                        data = os.path.join(self.dir_path, data)
                     metadata[tag] = data.decode(locale.getpreferredencoding())
                     break
 
         return metadata
 
-    def get(self, subdir = None):
+    def get_file_metadata(self, metadata, subdir=None):
+        '''
+        Returns the file metadata that could be found in the directory.
+        '''
+
+        if subdir is None: subdir = self.dir_path
+
+        if 'album_name' not in metadata.keys():
+            fmd = FileMetadata(os.path.join(subdir, 'album-name')).contents()
+            if fmd is not None:
+                metadata['album_name'] = fmd
+
+        if 'album_description' not in metadata.keys():
+            fmd = FileMetadata(os.path.join(subdir, 'album-description')).contents()
+            if fmd is not None:
+                metadata['album_description'] = fmd
+
+        if 'album_picture' not in metadata.keys():
+            fmd = FileMetadata(os.path.join(subdir, 'album-picture')).contents(splitter='\n')
+            if fmd is not None:
+                metadata['album_picture'] = fmd[0]
+
+        return metadata
+
+    def get(self, subdir=None, dir=None):
         '''
         Returns directory meta data. First tries to parse known formats
         and then fall backs to built in defaults.
@@ -453,10 +410,15 @@ class DirectoryMetadata(make.FileSimpleDependency):
         except NoMetadata:
             pass
 
+        result = self.get_file_metadata(result, subdir)
+
         # Add album picture
         if not result.has_key('album_picture'):
             try:
-                picture = self.dir.get_all_medias_paths()[0]
+                if dir is not None:
+                    picture = dir.get_all_medias_paths()[0]
+                else:
+                    raise IndexError
             except IndexError:
                 picture = None
             if picture is not None:
@@ -464,6 +426,12 @@ class DirectoryMetadata(make.FileSimpleDependency):
 
         return result
 
+    def get_title(self):
+        try:
+            return self.get()['album_name']
+        except KeyError:
+            return os.path.basename(self.dir_path).replace('_', ' ')
+
 
 class DefaultMetadata(make.FileMakeObject):
     """
@@ -479,9 +447,9 @@ class DefaultMetadata(make.FileMakeObject):
         self.album = album
 
     def build(self):
-        md = DirectoryMetadata(self.source_dir)
+        md = DirectoryMetadata(self.source_dir.path)
 
-        md_data = md.get()
+        md_data = md.get(None, self.source_dir)
         if 'album_description' in md_data.keys()\
         or 'album_name' in md_data.keys():
             self.album.log(_("  SKIPPED because metadata exists."))
